@@ -1,37 +1,36 @@
-from astropy.time import TimeUnixTai, TimeFromEpoch
 import astropy
+from astropy.time import TimeUnixTai, TimeFromEpoch
 
-from . import reco
-from . import io
-from . import visualization
 from . import calib
-from . import mc
-from . import spectra
+from . import data
+from . import high_level
 from . import image
-from . import irf
-
+from . import io
+from . import mc
+from . import reco
+from . import spectra
+from . import visualization
 from .io import standard_config
-from .version import get_version
+from .version import __version__
 
 __all__ = [
-    "reco",
-    "io",
-    "irf",
-    "visualization",
-    "calib",
-    "mc",
-    "spectra",
-    "image",
-    "standard_config",
     "__version__",
+    "data",
+    "high_level",
+    "calib",
+    "image",
+    "io",
+    "mc",
+    "reco",
+    "spectra",
+    "standard_config",
+    "visualization",
 ]
 
-__version__ = get_version(pep440=False)
-
 if (
-    astropy.version.major == 4
-    and astropy.version.minor <= 2
-    and astropy.version.bugfix <= 0
+        astropy.version.major == 4
+        and astropy.version.minor <= 2
+        and astropy.version.bugfix <= 0
 ):
     # clear the cache to not depend on import orders
     TimeFromEpoch.__dict__["_epoch"]._cache.clear()
