@@ -7,6 +7,7 @@ from astropy import units as u
 from ctapipe.calib.camera.flatfield import FlatFieldCalculator
 from ctapipe.core.traits import  List, Path
 from lstchain.calib.camera.time_sampling_correction import TimeSamplingCorrection
+from ctapipe.image.extractor import ImageExtractor
 
 __all__ = [
     'FlasherFlatFieldCalculator'
@@ -43,6 +44,8 @@ class FlasherFlatFieldCalculator(FlatFieldCalculator):
     ).tag(config=True)
 
     time_sampling_correction_path = Path(
+        default_value=None,
+        allow_none=True,
         exists=True, directory_ok=False,
         help='Path to time sampling correction file'
     ).tag(config=True)
@@ -86,6 +89,10 @@ class FlasherFlatFieldCalculator(FlatFieldCalculator):
         else:
             self.time_sampling_corrector = None
 
+        # fix for broken extractor setup in ctapipe baseclass
+        self.extractor = ImageExtractor.from_name(
+            self.charge_product, parent=self, subarray=subarray
+        )
 
     def _extract_charge(self, event):
         """
@@ -115,8 +122,6 @@ class FlasherFlatFieldCalculator(FlatFieldCalculator):
         if self.extractor:
             charge, peak_pos = self.extractor(waveforms, self.tel_id, no_gain_selection)
 
-
-
         # shift the time if time shift is already defined
         # (e.g. drs4 waveform time shifts for LST)
         time_shift = event.calibration.tel[self.tel_id].dl1.time_shift
@@ -278,9 +283,9 @@ class FlasherFlatFieldCalculator(FlatFieldCalculator):
                                              pixel_median > self.time_cut_outliers[1])
 
         return {
-            'sample_time': (trigger_time - time_start).value / 2 *u.s,
-            'sample_time_min': time_start.value*u.s,
-            'sample_time_max': trigger_time.value*u.s,
+            'sample_time': (time_start +(trigger_time - time_start) / 2).unix*u.s,
+            'sample_time_min': time_start.unix*u.s,
+            'sample_time_max': trigger_time.unix*u.s,
             'time_mean': np.ma.getdata(pixel_mean)*u.ns,
             'time_median': np.ma.getdata(pixel_median)*u.ns,
             'time_std': np.ma.getdata(pixel_std)*u.ns,
