@@ -4,11 +4,12 @@ Functions to check the contents of LST DL1 files and associated muon ring files
 
 __all__ = [
     'check_dl1',
-    'process_dl1_file',
+    'merge_dl1datacheck_files',
     'plot_datacheck',
-    'plot_trigger_types',
     'plot_mean_and_stddev',
-    'merge_dl1datacheck_files'
+    'plot_trigger_types',
+    'process_dl1_file',
+    'write_error_page',
 ]
 
 import logging
@@ -23,7 +24,6 @@ import matplotlib.dates as dates
 import matplotlib.pyplot as plt
 import matplotlib.ticker as mtick
 import numpy as np
-import pandas as pd
 import tables
 from astropy import units as u
 from astropy.table import Table, vstack
@@ -31,20 +31,24 @@ from ctapipe.containers import EventType
 from ctapipe.coordinates import EngineeringCameraFrame
 from ctapipe.instrument import SubarrayDescription
 from ctapipe.io import HDF5TableWriter
+from ctapipe.io import read_table
+from ctapipe_io_lst import TriggerBits
 from ctapipe.visualization import CameraDisplay
 
-from ctapipe_io_lst import TriggerBits
-
-# from lstchain.visualization.bokeh import plot_mean_and_stddev_bokeh
-# from bokeh.models.widgets import Panel
 from matplotlib.backends.backend_pdf import PdfPages
 from scipy.stats import poisson, sem
 
-from lstchain.datachecks.containers import DL1DataCheckContainer
-from lstchain.datachecks.containers import DL1DataCheckHistogramBins
-from lstchain.io.io import dl1_params_lstcam_key
-from lstchain.paths import parse_datacheck_dl1_filename, parse_dl1_filename, \
-    run_to_muon_filename, run_to_datacheck_dl1_filename
+from lstchain.datachecks.containers import (
+    DL1DataCheckContainer,
+    DL1DataCheckHistogramBins,
+)
+from lstchain.io.io import dl1_params_lstcam_key, dl1_images_lstcam_key
+from lstchain.paths import (
+    parse_datacheck_dl1_filename,
+    parse_dl1_filename,
+    run_to_muon_filename,
+    run_to_datacheck_dl1_filename,
+)
 
 
 def check_dl1(filenames, output_path, max_cores=4, create_pdf=False, batch=False):
@@ -112,6 +116,7 @@ def check_dl1(filenames, output_path, max_cores=4, create_pdf=False, batch=False
                               root_uep='/dl1/event/telescope/parameters') as f:
             for name in trig_tags:
                 trig_tags[name].extend(f.root.LST_LSTCam.col(name))
+
     num_pedestals = {'trigger_type':
                          (np.array(trig_tags['trigger_type']) ==
                           TriggerBits.PEDESTAL.value).sum(),
@@ -225,90 +230,89 @@ def process_dl1_file(filename, bins, tel_id=1):
     equivalent_focal_length = subarray_info.tel[tel_id].optics.equivalent_focal_length
     m2deg = np.rad2deg(u.m / equivalent_focal_length * u.rad) / u.m
 
-    with tables.open_file(filename) as file:
-        # unfortunately pandas.read_hdf does not seem compatible with
-        # 'with... as...' statements
-        parameters = pd.read_hdf(filename, key=dl1_params_lstcam_key)
+    parameters = read_table(filename, dl1_params_lstcam_key)
 
-        # convert cog distance to camera center from meters to degrees:
-        parameters['r'] *= m2deg
-        # time gradient from ns/m to ns/deg
-        parameters['time_gradient'] /= m2deg
+    # convert cog distance to camera center from meters to degrees:
+    parameters['r'] = parameters['r'].quantity * m2deg
+    # time gradient from ns/m to ns/deg
+    parameters['time_gradient'] = \
+        parameters['time_gradient'].quantity / m2deg
 
-        # We do not convert the x,y, cog coordinates, because only in m can
-        # CameraGeometry find the pixel where a given cog falls
+    image_table = read_table(filename, dl1_images_lstcam_key)
+    # create flatfield mask from the images table. For the time being,
+    # trigger type tags are not reliable. We first identify flatfield events
+    # by their looks.
+    flatfield_mask = (parameters['event_type'] == EventType.FLATFIELD.value)
+    # The same mask should be valid for image_table, since the entry in
+    # the two tables correspond one to one.
 
-        # in order to read in the images we have to use tables,
-        # because pandas is not compatible with vector columns
-        image_table = file.root.dl1.event.telescope.image.LST_LSTCam
+    # Revise flatfield_mask : event_type can be rarely wrong, so check
+    # here that all events look like interleaved flat field events:
+    # Note (AM, 20211202): finally we won't use this, the event source
+    # takes care of it
+    # flatfield_mask &= ((parameters['intensity'] > 50000) &
+    #                    (parameters['concentration_pixel'] < 0.005))
 
-        # create flatfield mask from the images table. For the time being,
-        # trigger type tags are not reliable. We first identify flatfield events
-        # by their looks.
-        flatfield_mask = (parameters['event_type'] == EventType.FLATFIELD.value)
-        # The same mask should be valid for image_table, since the entry in
-        # the two tables correspond one to one.
+    pedestal_mask = (parameters['event_type'] ==
+                     EventType.SKY_PEDESTAL.value)
 
-        # Revise flatfield_mask : event_type can be rarely wrong, so check
-        # here that all events look like interleaved flat field events:
-        # Note (AM, 20211202): finally we won't use this, the event source
-        # takes care of it
-        # flatfield_mask &= ((parameters['intensity'] > 50000) &
-        #                    (parameters['concentration_pixel'] < 0.005))
+    # Now obtain by exclusion the masks for cosmics:
+    cosmics_mask = ~(pedestal_mask | flatfield_mask)
 
-        pedestal_mask = (parameters['event_type'] ==
-                         EventType.SKY_PEDESTAL.value)
+    logger.info(f'   pedestals: {np.sum(pedestal_mask)}, '
+                f' flatfield: {np.sum(flatfield_mask)}, '
+                f' cosmics: {np.sum(cosmics_mask)}')
 
-        # Now obtain by exclusion the masks for cosmics:
-        cosmics_mask = ~(pedestal_mask | flatfield_mask)
+    # Fill quantities which depend on event-wise (i.e. not
+    # pixel-wise) parameters.
+    # Set None for a container that has not been filled,
+    # otherwise it will give trouble in the plotting stage.
 
-        logger.info(f'   pedestals: {np.sum(pedestal_mask)}, '
-                    f' flatfield: {np.sum(flatfield_mask)}, '
-                    f' cosmics: {np.sum(cosmics_mask)}')
+    if pedestal_mask.sum() > 1:
+        dl1datacheck_pedestals.fill_event_wise_info(subrun_index,
+                                                    parameters,
+                                                    pedestal_mask,
+                                                    geom, bins)
+        dl1datacheck_pedestals.fill_pixel_wise_info(image_table,
+                                                    pedestal_mask, bins,
+                                                    equivalent_focal_length,
+                                                    geom,
+                                                    'pedestals')
+    else:
+        dl1datacheck_pedestals = None
 
-        # Fill quantities which depend on event-wise (i.e. not
-        # pixel-wise) parameters.
-        # Set None for a container that has not been filled,
-        # otherwise it will give trouble in the plotting stage.
+    if flatfield_mask.sum() > 1:
+        dl1datacheck_flatfield.fill_event_wise_info(subrun_index,
+                                                    parameters,
+                                                    flatfield_mask,
+                                                    geom, bins)
+        dl1datacheck_flatfield.fill_pixel_wise_info(image_table,
+                                                    flatfield_mask, bins,
+                                                    equivalent_focal_length,
+                                                    geom,
+                                                    'flatfield')
+    else:
+        dl1datacheck_flatfield = None
 
-        if pedestal_mask.sum() > 1:
-            dl1datacheck_pedestals.fill_event_wise_info(subrun_index,
-                                                        parameters,
-                                                        pedestal_mask,
-                                                        geom, bins)
-            dl1datacheck_pedestals.fill_pixel_wise_info(image_table,
-                                                        pedestal_mask, bins,
-                                                        'pedestals')
-        else:
-            dl1datacheck_pedestals = None
+    if cosmics_mask.sum() > 1:
+        dl1datacheck_cosmics.fill_event_wise_info(subrun_index,
+                                                  parameters,
+                                                  cosmics_mask,
+                                                  geom, bins)
+        dl1datacheck_cosmics.fill_pixel_wise_info(image_table,
+                                                  cosmics_mask, bins,
+                                                  equivalent_focal_length,
+                                                  geom,
+                                                  'cosmics')
+    else:
+        dl1datacheck_cosmics = None
 
-        if flatfield_mask.sum() > 1:
-            dl1datacheck_flatfield.fill_event_wise_info(subrun_index,
-                                                        parameters,
-                                                        flatfield_mask,
-                                                        geom, bins)
-            dl1datacheck_flatfield.fill_pixel_wise_info(image_table,
-                                                        flatfield_mask, bins,
-                                                        'flatfield')
-        else:
-            dl1datacheck_flatfield = None
-
-        if cosmics_mask.sum() > 1:
-            dl1datacheck_cosmics.fill_event_wise_info(subrun_index,
-                                                      parameters,
-                                                      cosmics_mask,
-                                                      geom, bins)
-            dl1datacheck_cosmics.fill_pixel_wise_info(image_table,
-                                                      cosmics_mask, bins,
-                                                      'cosmics')
-        else:
-            dl1datacheck_cosmics = None
-
-        return dl1datacheck_pedestals, dl1datacheck_flatfield, \
-               dl1datacheck_cosmics
+    return dl1datacheck_pedestals, dl1datacheck_flatfield, \
+           dl1datacheck_cosmics
 
 
-def plot_datacheck(datacheck_filename, out_path=None, batch=False, muons_dir=None, tel_id=1):
+def plot_datacheck(datacheck_filename, out_path=None, batch=False,
+                   muons_dir=None, create_pdf=True, tel_id=1):
     """
     Plot datacheck
 
@@ -348,6 +352,10 @@ def plot_datacheck(datacheck_filename, out_path=None, batch=False, muons_dir=Non
             # just a single .h5 file:
             datacheck_filename = datacheck_filename[0]
 
+    if not create_pdf:
+        # Nothing else to be done:
+        return
+
     pdf_filename = Path(datacheck_filename).with_suffix('.pdf')
     # set output directory if provided:
     if out_path is not None:
@@ -857,7 +865,7 @@ def plot_datacheck(datacheck_filename, out_path=None, batch=False, muons_dir=Non
                 num_rings = np.append(num_rings, 0)
                 num_contained_rings = np.append(num_contained_rings, 0)
 
-        if len(files_with_rings) == 0:
+        if len(good_files) == 0:
             write_error_page('Muons', pagesize)
             pdf.savefig()
             return
@@ -1186,7 +1194,7 @@ def merge_dl1datacheck_files(file_list):
 
     first_file_name = file_list[0]
     first_file = tables.open_file(first_file_name)
-    #  get run number and build the name of the merged file:
+    # get run number and build the name of the merged file:
     file = parse_datacheck_dl1_filename(os.path.basename(first_file_name))
     merged_filename = run_to_datacheck_dl1_filename(file.tel_id, file.run,
                                                     None, None)
