@@ -1,13 +1,17 @@
+import numpy as np
+import operator
+import astropy.units as u
+
+from ctapipe.containers import EventType
 from ctapipe.core import Component
 from ctapipe.core.traits import Dict, List, Float, Int
 from lstchain.reco.utils import filter_events
 
-import numpy as np
-import astropy.units as u
 from pyirf.binning import create_bins_per_decade  # , add_overflow_bins
+from pyirf.cuts import calculate_percentile_cut, evaluate_binned_cut
 
 
-__all__ = ["EventSelector", "DL3FixedCuts", "DataBinning"]
+__all__ = ["EventSelector", "DL3Cuts", "DataBinning"]
 
 
 class EventSelector(Component):
@@ -15,6 +19,10 @@ class EventSelector(Component):
     Filter values used for event filters and list of finite parameters are
     taken as inputs and filter_events() is used on a table of events
     called in with the Component.
+
+    For event_type, we choose the sub-array trigger, EventType.SUBARRAY.value,
+    32, which is for shower event candidate, as per the latest CTA R1 Event
+    Data Model.
     """
 
     filters = Dict(
@@ -23,6 +31,7 @@ class EventSelector(Component):
             "r": [0, 1],
             "wl": [0.01, 1],
             "leakage_intensity_width_2": [0, 1],
+            "event_type": [EventType.SUBARRAY.value, EventType.SUBARRAY.value],
         },
     ).tag(config=True)
 
@@ -32,37 +41,147 @@ class EventSelector(Component):
     ).tag(config=True)
 
     def filter_cut(self, events):
+        """
+        Apply the event filters
+        """
         return filter_events(events, self.filters, self.finite_params)
 
 
-class DL3FixedCuts(Component):
+class DL3Cuts(Component):
     """
-    Temporary fixed selection cuts for DL2 to DL3 conversion
+    Selection cuts for DL2 to DL3 conversion
     """
 
-    fixed_gh_cut = Float(
-        help="Fixed selection cut for gh_score (gammaness)",
+    global_gh_cut = Float(
+        help="Global selection cut for gh_score (gammaness)",
         default_value=0.6,
     ).tag(config=True)
 
-    fixed_theta_cut = Float(
-        help="Fixed selection cut for theta",
+    gh_efficiency = Float(
+        help="Gamma efficiency for optimized g/h cuts in %",
+        default_value=0.95,
+    ).tag(config=True)
+
+    theta_containment = Float(
+        help="Percentage containment region for theta cuts",
+        default=0.68,
+    ).tag(config=True)
+
+    global_theta_cut = Float(
+        help="Global selection cut for theta",
         default_value=0.2,
     ).tag(config=True)
 
+    global_alpha_cut = Float(
+        help="Global selection cut for alpha",
+        default_value=20,
+    ).tag(config=True)
+
     allowed_tels = List(
         help="List of allowed LST telescope ids",
         trait=Int(),
         default_value=[1],
     ).tag(config=True)
 
-    def gh_cut(self, data):
-        return data[data["gh_score"] > self.fixed_gh_cut]
+    def apply_global_gh_cut(self, data):
+        """
+        Applying a global gammaness cut on a given data
+        """
+        return data[data["gh_score"] > self.global_gh_cut]
 
-    def theta_cut(self, data):
-        return data[data["theta"].to_value(u.deg) < self.fixed_theta_cut]
+    def energy_dependent_gh_cuts(
+        self, data, energy_bins, min_value=0.1,
+        max_value=0.99, smoothing=None, min_events=10
+    ):
+        """
+        Evaluating energy-dependent gammaness cuts, in a given
+        data, with provided reco energy bins, and other parameters to
+        pass to the pyirf.cuts.calculate_percentile_cut function
+        """
+
+        gh_cuts = calculate_percentile_cut(
+            data["gh_score"],
+            data["reco_energy"],
+            bins=energy_bins,
+            min_value=min_value,
+            max_value=max_value,
+            fill_value=data["gh_score"].max(),
+            percentile=100 * (1 - self.gh_efficiency),
+            smoothing=smoothing,
+            min_events=min_events,
+        )
+        return gh_cuts
+
+    def apply_global_alpha_cut(self, data):
+        """
+        Applying a global alpha cut on a given data
+        """
+        return data[data["alpha"].to_value(u.deg) < self.global_alpha_cut]
+
+    def apply_energy_dependent_gh_cuts(self, data, gh_cuts):
+        """
+        Applying a given energy-dependent gh cuts to a data file, along the reco
+        energy bins provided.
+        """
+
+        data["selected_gh"] = evaluate_binned_cut(
+            data["gh_score"],
+            data["reco_energy"],
+            gh_cuts,
+            operator.ge,
+        )
+        return data[data["selected_gh"]]
+
+    def apply_global_theta_cut(self, data):
+        """
+        Applying a global theta cut on a given data
+        """
+        return data[data["theta"].to_value(u.deg) < self.global_theta_cut]
+
+    def energy_dependent_theta_cuts(
+        self, data, energy_bins, min_value=0.05 * u.deg,
+        fill_value=0.32 * u.deg, max_value=0.32 * u.deg,
+        smoothing=None, min_events=10
+    ):
+        """
+        Evaluating an optimized energy-dependent theta cuts, in a given
+        data, with provided reco energy bins, and other parameters to
+        pass to the pyirf.cuts.calculate_percentile_cut function.
+
+        Note: Using too fine binning will result in too un-smooth cuts.
+        """
+
+        theta_cuts = calculate_percentile_cut(
+            data["theta"],
+            data["reco_energy"],
+            bins=energy_bins,
+            min_value=min_value,
+            max_value=max_value,
+            fill_value=fill_value,
+            percentile=100 * self.theta_containment,
+            smoothing=smoothing,
+            min_events=min_events,
+        )
+        return theta_cuts
+
+    def apply_energy_dependent_theta_cuts(self, data, theta_cuts):
+        """
+        Applying a given energy-dependent theta cuts to a data file, along the
+        reco energy bins provided.
+        """
+
+        data["selected_theta"] = evaluate_binned_cut(
+            data["theta"],
+            data["reco_energy"],
+            theta_cuts,
+            operator.le,
+        )
+        return data[data["selected_theta"]]
 
     def allowed_tels_filter(self, data):
+        """
+        Applying a filter on telescopes used for observation.
+        """
         mask = np.zeros(len(data), dtype=bool)
         for tel_id in self.allowed_tels:
             mask |= data["tel_id"] == tel_id
@@ -87,7 +206,7 @@ class DataBinning(Component):
 
     true_energy_n_bins_per_decade = Float(
         help="Number of edges per decade for True Energy bins",
-        default_value=5.5,
+        default_value=5,
     ).tag(config=True)
 
     reco_energy_min = Float(
@@ -102,7 +221,7 @@ class DataBinning(Component):
 
     reco_energy_n_bins_per_decade = Float(
         help="Number of edges per decade for Reco Energy bins",
-        default_value=5.5,
+        default_value=5,
     ).tag(config=True)
 
     energy_migration_min = Float(
@@ -168,9 +287,13 @@ class DataBinning(Component):
     def true_energy_bins(self):
         """
         Creates bins per decade for true MC energy using pyirf function.
+        The overflow binning added is not needed at the current stage.
 
-        The overflow binning added is not needed at the current stage
-        It can be used as - add_overflow_bins(***)[1:-1]
+        Examples
+        --------
+        It can be used as:
+
+        >>> add_overflow_bins(***)[1:-1]
         """
         true_energy = create_bins_per_decade(
             self.true_energy_min * u.TeV,
@@ -182,9 +305,13 @@ class DataBinning(Component):
     def reco_energy_bins(self):
         """
         Creates bins per decade for reconstructed MC energy using pyirf function.
+        The overflow binning added is not needed at the current stage.
 
-        The overflow binning added is not needed at the current stage
-        It can be used as - add_overflow_bins(***)[1:-1]
+        Examples
+        --------
+        It can be used as:
+
+        >>> add_overflow_bins(***)[1:-1]
         """
         reco_energy = create_bins_per_decade(
             self.reco_energy_min * u.TeV,
@@ -206,7 +333,7 @@ class DataBinning(Component):
 
     def fov_offset_bins(self):
         """
-        Creates bins for single/multiple FoV offset
+        Creates bins for single/multiple FoV offset.
         """
         fov_offset = (
             np.linspace(
