@@ -30,7 +30,6 @@ from lstchain.image.muon import (
     tag_pix_thr,
 )
 from lstchain.io.io import dl1_params_lstcam_key, dl1_images_lstcam_key
-from lstchain.io.io import read_telescopes_descriptions, read_subarray_description
 from lstchain.visualization import plot_calib
 
 parser = argparse.ArgumentParser()
@@ -57,7 +56,6 @@ parser.add_argument(
     help="Path to corresponding calibration file (containing bad pixel information).",
 )
 
-
 # Optional argument
 parser.add_argument(
     "--plot-rings",
@@ -82,8 +80,9 @@ parser.add_argument(
 
 
 def main():
+
     args = parser.parse_args()
-    
+
     print("input files: {}".format(args.input_file))
     print("calib file: {}".format(args.calib_file))
     print("output file: {}".format(args.output_file))
@@ -114,36 +113,37 @@ def main():
     for filename in filenames:
         print('Opening file', filename)
 
-        subarray_info = SubarrayDescription.from_hdf(filename)
-        geom = subarray_info.tel[lst1_tel_id].camera.geometry
-
-        subarray = read_subarray_description(filename, subarray_name='LST-1')
-
-        images = Table.read(filename, path=dl1_images_lstcam_key)['image']
-
-        parameters = pd.read_hdf(filename, key=dl1_params_lstcam_key)
-        telescope_description = read_telescopes_descriptions(filename)[lst1_tel_id]
-
+        subarray = SubarrayDescription.from_hdf(filename)
+        telescope_description = subarray.tel[lst1_tel_id]
+        geom = telescope_description.camera.geometry
         equivalent_focal_length = telescope_description.optics.equivalent_focal_length
         mirror_area = telescope_description.optics.mirror_area
 
+        images = Table.read(filename, path=dl1_images_lstcam_key)['image']
+        parameters = pd.read_hdf(filename, key=dl1_params_lstcam_key)
+
+
         # fill dummy event times with NaNs in case they do not exist (like in MC):
         if 'dragon_time' not in parameters.keys():
             dummy_times = np.empty(len(parameters['event_id']))
             dummy_times[:] = np.nan
             parameters['dragon_time'] = dummy_times
 
+        # fill MC energies with -1 in case they do not exist (like in real data):
+        if 'mc_energy' not in parameters.keys():
+            parameters['mc_energy'] = np.ones_like(parameters['event_id']) * -1
+
         for full_image, event_id, dragon_time, mc_energy in zip(
                 images, parameters['event_id'], parameters['dragon_time'], parameters['mc_energy']):
             if args.calib_file is not None:
-                image = full_image*(~bad_pixels)
+                image = full_image * (~bad_pixels)
             else:
                 image = full_image
             # print("Event {}. Number of pixels above 10 phe: {}".format(event_id,
             #                                                           np.size(image[image > 10.])))
             # if((np.size(image[image > 10.]) > 300) or (np.size(image[image > 10.]) < 50)):
             #     continue
-            if not tag_pix_thr(image): # default skips pedestal and calibration events
+            if not tag_pix_thr(image):  # default skips pedestal and calibration events
                 continue
 
             # default values apply no filtering.
@@ -156,9 +156,9 @@ def main():
                 muonringparam, good_ring, radial_distribution,
                 mean_pixel_charge_around_ring, muonparameters
             ) = analyze_muon_event(subarray,
-                event_id, image, geom, equivalent_focal_length,
-                mirror_area, args.plot_rings, args.plots_path
-            )
+                                   event_id, image, geom, equivalent_focal_length,
+                                   mirror_area, args.plot_rings, args.plots_path
+                                   )
 
             if good_ring:
                 num_muons += 1
@@ -183,5 +183,6 @@ def main():
     table = Table(output_parameters)
     table.write(args.output_file, format='fits', overwrite=True)
 
+
 if __name__ == '__main__':
     main()
